(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
	(factory((global.reshep = global.reshep || {}),global.React));
}(this, (function (exports,React) { 'use strict';

React = React && 'default' in React ? React['default'] : React;

var createElement = function createElement(type, props, children) {
  return React.createElement(type, props, children);
};

var createFactory = function createFactory(type) {
  return function (props, children) {
    return createElement(type, props, children);
  };
};

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};





















var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();











var toArray = function (arr) {
  return Array.isArray(arr) ? arr : Array.from(arr);
};

var branch = function branch(predicate) {
  return function (left) {
    return function (right) {
      return function () {
        return predicate.apply(undefined, arguments) ? right.apply(undefined, arguments) : left.apply(undefined, arguments);
      };
    };
  };
};

var compose = function compose() {
  for (var _len = arguments.length, fs = Array(_len), _key = 0; _key < _len; _key++) {
    fs[_key] = arguments[_key];
  }

  var gs = fs.reverse();

  return function (x) {
    return gs.reduce(function (y, f) {
      return f(y);
    }, x);
  };
};

var deepMerge = function deepMerge() {
  for (var _len2 = arguments.length, srcs = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    srcs[_key2] = arguments[_key2];
  }

  return compose(reduce(function (targ, src) {
    return reduce(function (t, _ref) {
      var _ref2 = slicedToArray(_ref, 2),
          k = _ref2[0],
          v = _ref2[1];

      t[k] = isObject(t[k]) && isObject(v) ? deepMerge(t[k], v) : v;

      return t;
    })(targ)(src);
  })({}), map(entries))(srcs);
};

var entries = function entries(src) {
  return Object.keys(src).map(function (key) {
    return [key, src[key]];
  });
};

var getPath = function getPath(_ref3, src) {
  var _ref4 = toArray(_ref3),
      key = _ref4[0],
      path = _ref4.slice(1);

  return path.length < 1 ? src[key] : getPath(path, src[key]);
};

var isObject = function isObject(x) {
  return typeOf(x) === "object";
};

var map = function map(f) {
  return function (xs) {
    return xs.map(f);
  };
};

var reduce = function reduce(f) {
  return function (init) {
    return function (xs) {
      return xs.reduce(f, init);
    };
  };
};

var set = function set(key, value, target) {
  return Object.assign({}, target, defineProperty({}, key, value));
};

var setPath = function setPath(_ref5, value, target) {
  var _ref6 = toArray(_ref5),
      key = _ref6[0],
      path = _ref6.slice(1);

  return path.length < 1 ? set(key, value, target) : _extends({}, target, defineProperty({}, key, setPath(path, value, target.hasOwnProperty(key) ? target[key] : {})));
};

var split = function split(delimiter) {
  return function (s) {
    return s.split(delimiter);
  };
};

var typeOf = function typeOf(x) {
  return {}.toString.call(x).match(/\s([a-z|A-Z]+)/)[1].toLowerCase();
};

var splitPath = split('.');

var splitPaths = map(splitPath);

var getValue = function getValue(src) {
  return function (_ref) {
    var _ref2 = slicedToArray(_ref, 2),
        from = _ref2[0],
        to = _ref2[1];

    return [to, getPath(from, src)];
  };
};

var setValue = function setValue(targ, _ref3) {
  var _ref4 = slicedToArray(_ref3, 2),
      to = _ref4[0],
      val = _ref4[1];

  return setPath(to, val, targ);
};

var mapShape = function mapShape(pathMap) {
  var paths = entries(pathMap);

  return function (props) {
    return compose(reduce(setValue)({}), map(getValue(props)), map(splitPaths))(paths);
  };
};

var setStatic = function setStatic(key, value) {
  return function (BaseComponent) {
    BaseComponent[key] = value;

    return BaseComponent;
  };
};

var setDisplayName = function setDisplayName(displayName) {
  return setStatic('displayName', displayName);
};

var isValid = function isValid(input) {
  return isObject(input);
};

var paramName = '`pathMap`';

var handleInvalid = function handleInvalid(input) {
  throw new TypeError('Expected ' + paramName + ' to be an object. ' + (paramName + ' is type ' + typeOf(input) + ' instead.'));
};

var validateShapeMap = branch(isValid)(handleInvalid);

var getDisplayName = function getDisplayName(Component) {
  if (typeof Component === "string") {
    return Component;
  }

  if (!Component) {
    return undefined;
  }

  return Component.displayName || Component.name || "Component";
};

var wrapDisplayName = function wrapDisplayName(BaseComponent, hocName) {
  return hocName + '(' + getDisplayName(BaseComponent) + ')';
};

var shapeFrom = function shapeFrom(pathMap) {
  var mapper = mapShape(pathMap);

  return function (BaseComponent) {
    var factory = createFactory(BaseComponent);

    var ShapeFrom = function ShapeFrom(props) {
      return factory(mapper(props));
    };

    if (process.env.NODE_ENV !== 'production') {
      return setDisplayName(wrapDisplayName(BaseComponent, 'shapeFrom'))(ShapeFrom);
    }

    return ShapeFrom;
  };
};

var shapeFrom$1 = validateShapeMap(shapeFrom);

var shapeWith = function shapeWith(pathMap) {
  var mapper = mapShape(pathMap);

  return function (BaseComponent) {
    var factory = createFactory(BaseComponent);

    var ShapeWith = function ShapeWith(props) {
      return factory(deepMerge(props, mapper(props)));
    };

    if (process.env.NODE_ENV !== 'production') {
      return setDisplayName(wrapDisplayName(BaseComponent, 'shapeWith'))(ShapeWith);
    }

    return ShapeWith;
  };
};

var shapeWith$1 = validateShapeMap(shapeWith);

exports.shapeFrom = shapeFrom$1;
exports.shapeWith = shapeWith$1;

Object.defineProperty(exports, '__esModule', { value: true });

})));
